#!/usr/bin/perl

=head1 NAME

get-cuis.pl

=head1 SYNOPSIS

This package creates a directory of choice files containing 
the actual CUI from the 1999 Unified Medical Language System 
(UMLS) Metathesaurus for each of the possible concepts of the 
target words in the NLM-WSD dataset. 

=head1 DESCRIPTION

This program takes as input the location of the NLM-WSD 
Reviewed_Results directory, a destination directory, the 
MRCON file from the 1999 UMLS and the MRSTY file from the 
1999 UMLS and produces a directory containing the choice 
files for each of the target words in the NLM-WSD dataset.

=head1 USAGE

perl get-cuis.pl SOURCE DESTINATION MRCON MRSTY


=head1 USAGE Example:

Here is an example of the usage that I would run:

perl get-cuis.pl /home/NLM-WSD/Reviewed_Results/ choices /home/1999UMLS/META/MRCON /home/1999UMLS/META/MRSTY

=head1 INPUT

=head2 Required Arguments:

=head3 SOURCE

The Reviewed_Results directory from the NLM-WSD dataset.

=head3 DESTINATION

The directory that will contain the choice files for each of 
the target words in the NLM-WSD datset.

=head3 MRCON

The location of the MRCON table from the 1999 Unified Medical 
Language System (UMLS) Metathesaurus

=head3 MRSTY

The location of the MRSTY table fromt eh 1999 Unified Medical 
Language System (UMLS) Metathesaurus

=head2 Optional Arguments:

=head3 --help

Displays the quick summary of program options.

=head3 --version

Displays the version information.

=head1 OUTPUT

The directory containing the chioce files for each of 
the target words in the NLM-wSD data set is called: 
    
    DESTINATION

There exists a choice file for each target word in the NLM-WSD 
dataset. The format for each of the choice files is similar to 
the choice files in the Reviewed_Results directory of the NLM-WSD 
dataset. The format is as follows:

<TAG>|<TERM>|<Semantic Type>|<CUI>

For example, one of the possible concepts for the target word 
adjustment is:

Adjustment <1> (Individual Adjustment)|inbe, Individual Behavior|C0376209

The expected name of the file is <target word>.choices. For 
our adjustment example, the name of the file created is 
"adjustment.choices"

=head1 PROGRAM REQUIREMENTS

=over

=item * Perl (version 5.8.5 or better) - http://www.perl.org

=back

=head1 AUTHOR

 Bridget T. McInnes, University of Minnesota, Twin Cities

=head1 COPYRIGHT

Copyright (c) 2007-2008,

 Bridget T. McInnes, University of Minnesota, Twin Cities
 bthomson at cs.umn.edu

 Ted Pedersen, University of Minnesota Duluth
 tpederse at d.umn.edu

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to 

 The Free Software Foundation, Inc.,
 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA.

=cut

use Getopt::Long;
eval(GetOptions( "version", "help")) or die ("Please check the above mentioned option(s).\n");

#  if help is defined, print out help
if( defined $opt_help ) {
    $opt_help = 1;
    &showHelp();
    exit;
}

#  if version is requested, show version
if( defined $opt_version ) {
    $opt_version = 1;
    &showVersion();
    exit;
}

if($#ARGV < 3) {
    $opt_help = 1;
    print STDERR "SOURCE, DESTINATION, MCON AND MRSTY need to be defined\n";
    &minimalUsageNotes();
    exit;
}

#  NLM-WSD Reviewed_Results directory
my $source = shift; #"$ENV{NLMWSDHOME}/Reviewed_Results";

#  DESTINATION directory
my $dest = shift;   #"$ENV{HOME}/work/NLM-WSD.target_word.choices/choices";

#  The MRCON FILE
my $mrconFile = shift; #"/nfsvol/nlsaux5/nls/umls_on_aux5/UMLS1999/META/MRCON";

#  The MRSTY FILE
my $mrstyFile = shift; #"/nfsvol/nlsaux5/nls/umls_on_aux5/UMLS1999/META/MRSTY";

#  check that destination has been supplied
if( !($dest) ) {
    print STDERR "No output directory (DESTINATION) supplied.\n";
    &askHelp();
    exit;
}
 
if(! (-e $dest ) ) {
    system("mkdir $dest");
}


#  hash variables
my %mrcon  = ();
my %mrsty  = ();
my %num_st = ();

#  get the MRCON information
open(MRCON, $mrconFile) || die "Could not open MRCON file\n";
while(<MRCON>) {
    chomp;
    my @array = split/\|/;
    $mrcon{lc($array[6])}{$array[0]}++;
} close MRCON;

#  get the MRSTY information
open(MRSTY, $mrstyFile) || die "Could not open MRSTY file\n";
while(<MRSTY>) {
    chomp;
    my @array = split/\|/;
    $mrsty{$array[0]}{$array[2]}++;
    $num_st{$array[0]}++;
}

#  get the target words from the NLM-WSD dataset
opendir(DIR, $source) || die "Could not open source directory: $source\n";
my @files = grep { $_ ne '.' and $_ ne '..' and $_ ne "CVS" and $_ ne "raw_summary" and $_ ne "index.shtml"} readdir DIR;


foreach my $tw (@files) {

    my $file        = "$source/$tw/choices";    
    my $destination = "$dest/$tw.choices";
    
    open(DST, ">$destination") || die "FILE $destination\n";

    open(FILE, $file) || die "Could not open $file\n";

    while(<FILE>) {
	chomp;
	my($tag, $term, $st) = split/\|/;

	my $cui = "";
	
	#  set the possible term
	my $pterm = $term;
	$pterm=~s/\(.*?\)//g;
	$pterm=~s/[\(\)]//g;
	$pterm=~s/^\s+//g;
	$pterm=~s/\s+$//g;

	#  problem with spacing and number in the term
	$pterm=~s/([A-Za-z])(\<[0-9]\>)/$1 $2/;
	$pterm=~s/(NOS) \<[0-9]\>/NOS/;

	#  get the associated semantic types
	my %sthash = ();
	while($st=~/([a-z][a-z][a-z][a-z])\, ([A-Za-z\s\,]+)\;?/g) { 
	    my $possible_st = $2;
	    if($tw eq "lead" and $possible_st eq "elii, Element, Ion, or Isotope") {
		$possible_st=~s/elii\, //g;
	    }
	    $sthash{$possible_st}++; 
	}

	#  get the semantic types associated with the concept
	my $stkeys = keys %sthash;

	#  check the semantic types for each of the possible cuis 
	#  associated with this term when there is a match print
	foreach my $possible_cui (sort keys %{$mrcon{lc($pterm)}}) {
	    
	    if($stkeys != $num_st{$possible_cui}) { next; }

	    my $flag = 1; 
	    foreach my $possible_st (sort keys %sthash) {
		if($flag == 0) { next; }
		if(!(exists ($mrsty{$possible_cui}{$possible_st}))) { $flag = 0; }
	    }
	    if($flag == 1) {
		print DST "$tag|$term|$st|$possible_cui\n";
	    }
	}
    }   
}
	 
##############################################################################
#  SUB FUNCTIONS
##############################################################################

#  function to output minimal usage notes
sub minimalUsageNotes {
    
    print STDERR "Usage: get-cuis.pl [OPTIONS] SOURCE DESTINATION MRCON MRSTY\n";
    print STDERR "Type get-cuis.pl --help for help.\n\n";
}

#  function to output help messages for this program
sub showHelp() {

    print "Usage: get-cuis.pl [OPTIONS} SOURCE DESTINATION MRCON MRSTY\n";
    

    print "OPTIONS:\n\n";

    print "--version                Prints the version number\n\n";

    print "--help                   Prints this help message.\n\n";
}

#  function to output the version number
sub showVersion {
        print '$Id: get-cuis.pl,v 0.01 2008/07/02 22:05:16 btmcinnes Exp $';
        print "\nCopyright (c) 2008, Ted Pedersen & Bridget McInnes\n";
}

#  function to output "ask for help" message when user's goofed
sub askHelp {
    print STDERR "Type get-cuis.pl --help for help.\n";
}
    
